package com.alicom.dycdp.api;

import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.dycdpapi.model.v20170525.*;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;

import java.util.UUID;

/**
 * Created on 17/6/10.
 * 流量API产品的DEMO程序,工程中包含了一个CdpDemo类，直接通过
 * 执行main函数即可体验流量产品API功能(只需要将AK替换成开通了云通信-流量产品功能的AK即可)
 * 工程依赖了2个jar包(存放在工程的libs目录下)
 * 1:aliyun-java-sdk-core.jar
 * 2:aliyun-java-sdk-dycdpapi.jar
 *
 * 备注:Demo工程编码采用UTF-8
 */
public class CdpDemo {

    //产品名称:云通信短信API产品,开发者无需替换
    static final String product = "Dycdpapi";
    //产品域名,开发者无需替换
    static final String domain = "dycdpapi.aliyuncs.com";

    //TODO 此处需要替换成开发者自己的AK(在阿里云访问控制台寻找)
    static final String accessKeyId = "yourAccessKeyId";
    static final String accessKeySecret = "yourAccessKeySecret";

    /**
     * 流量充值
     * @param phoneNumber
     * @param outId
     * @return
     * @throws ClientException
     */
    public static ChargeResponse charge(String phoneNumber, String outId) throws ClientException {

        //可自助调整超时时间
        System.setProperty("sun.net.client.defaultConnectTimeout", "10000");
        System.setProperty("sun.net.client.defaultReadTimeout", "10000");

        //初始化acsClient,暂不支持region化
        IClientProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKeyId, accessKeySecret);
        DefaultProfile.addEndpoint("cn-hangzhou", "cn-hangzhou", product, domain);
        IAcsClient acsClient = new DefaultAcsClient(profile);

        //组装请求对象-具体描述见控制台-文档部分内容
        ChargeRequest request = new ChargeRequest();
        //必填-待充值的号码
        request.setPhoneNumber(phoneNumber);
        //必填-充值档位,需要先通过档位查询接口查询当前待充值手机号码所支持的档位
        request.setGrade("300");
        //可选-是否分省默认为false
        request.setIsProvince("false");
        //可选-0全国流量 1省内流量-默认不填为0
        request.setScope("0");
        //可选-充值备注
        request.setReason("remark");
        //必填-调用方提供的充值流水号,服务提供方针对同一流水号只做一次充值动作。
        request.setOutId(outId);

        //hint 此处可能会抛出异常，注意catch
        ChargeResponse chargeResponse = acsClient.getAcsResponse(request);

        return chargeResponse;

    }

    /**
     * 流量档位查询
     * @param phoneNumber
     * @return
     * @throws ClientException
     */
    public static QueryGradesResponse queryGrades(String phoneNumber) throws ClientException {

        //可自助调整超时时间
        System.setProperty("sun.net.client.defaultConnectTimeout", "10000");
        System.setProperty("sun.net.client.defaultReadTimeout", "10000");

        //初始化acsClient,暂不支持region化
        IClientProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKeyId, accessKeySecret);
        DefaultProfile.addEndpoint("cn-hangzhou", "cn-hangzhou", product, domain);
        IAcsClient acsClient = new DefaultAcsClient(profile);

        //组装请求对象
        QueryGradesRequest request = new QueryGradesRequest();
        //可选-待充值的号码,如果不填,会返回各个运营商支持的流量档位情况
        request.setPhoneNumber(phoneNumber);
        //hint 此处可能会抛出异常，注意catch
        QueryGradesResponse queryGradesResponse = acsClient.getAcsResponse(request);

        return queryGradesResponse;

    }


    public static QueryChargeDetailResponse queryChargeDetail(String outId) throws ClientException {

        //可自助调整超时时间
        System.setProperty("sun.net.client.defaultConnectTimeout", "10000");
        System.setProperty("sun.net.client.defaultReadTimeout", "10000");

        //初始化acsClient,暂不支持region化
        IClientProfile profile = DefaultProfile.getProfile("cn-hangzhou", accessKeyId, accessKeySecret);
        DefaultProfile.addEndpoint("cn-hangzhou", "cn-hangzhou", product, domain);
        IAcsClient acsClient = new DefaultAcsClient(profile);

        QueryChargeDetailRequest request = new QueryChargeDetailRequest();
        //必填-调用方提供的充值流水号
        request.setOutId(outId);

        //hint 此处可能会抛出异常，注意catch
        QueryChargeDetailResponse queryChargeDetailResponse = acsClient.getAcsResponse(request);

        return queryChargeDetailResponse;

    }

    public static void main(String[] args) throws ClientException {

        String phoneNumber = "15000000000";
        QueryGradesResponse queryGradesResponse = queryGrades(phoneNumber);
        System.out.println("查询流量档位--------------");
        System.out.println("RequestId=" + queryGradesResponse.getRequestId());
        System.out.println("Code=" + queryGradesResponse.getCode());
        System.out.println("Message=" + queryGradesResponse.getMessage());
        System.out.println("Grades=" + queryGradesResponse.getGrades());

        String outId = UUID.randomUUID().toString();
        ChargeResponse chargeResponse = charge(phoneNumber, outId);
        System.out.println("充值流量--------------");
        System.out.println("RequestId=" + chargeResponse.getRequestId());
        System.out.println("Code=" + chargeResponse.getCode());
        System.out.println("Message=" + chargeResponse.getMessage());

        QueryChargeDetailResponse queryChargeDetailResponse = queryChargeDetail(outId);
        System.out.println("明细查询--------------");
        System.out.println("RequestId=" + queryChargeDetailResponse.getRequestId());
        System.out.println("Code=" + queryChargeDetailResponse.getCode());
        System.out.println("Message=" + queryChargeDetailResponse.getMessage());

    }
}

